@@ -32,7 +32,7 @@ class RateLimiter(object):
 
         """
         self.delay()
-        kwargs['headers'] = set_header_callback()
+        kwargs["headers"] = set_header_callback()
         response = request_function(*args, **kwargs)
         self.update(response.headers)
         return response
@@ -44,8 +44,9 @@ class RateLimiter(object):
         sleep_seconds = self.next_request_timestamp - time.time()
         if sleep_seconds <= 0:
             return
-        message = 'Sleeping: {:0.2f} seconds prior to' \
-                  ' call'.format(sleep_seconds)
+        message = "Sleeping: {:0.2f} seconds prior to" " call".format(
+            sleep_seconds
+        )
         log.debug(message)
         time.sleep(sleep_seconds)
 
@@ -59,28 +60,24 @@ class RateLimiter(object):
         responses should trigger exceptions that indicate invalid behavior.
 
         """
-        if 'x-ratelimit-remaining' not in response_headers:
+        if "x-ratelimit-remaining" not in response_headers:
             if self.remaining is not None:
                 self.remaining -= 1
                 self.used += 1
             return
 
         now = time.time()
-        prev_remaining = self.remaining
 
-        seconds_to_reset = int(response_headers['x-ratelimit-reset'])
-        self.remaining = float(response_headers['x-ratelimit-remaining'])
-        self.used = int(response_headers['x-ratelimit-used'])
+        seconds_to_reset = int(response_headers["x-ratelimit-reset"])
+        self.remaining = float(response_headers["x-ratelimit-remaining"])
+        self.used = int(response_headers["x-ratelimit-used"])
         self.reset_timestamp = now + seconds_to_reset
 
         if self.remaining <= 0:
             self.next_request_timestamp = self.reset_timestamp
             return
 
-        if prev_remaining is not None and prev_remaining > self.remaining:
-            estimated_clients = prev_remaining - self.remaining
-        else:
-            estimated_clients = 1.0
-
-        self.next_request_timestamp = min(self.reset_timestamp, now + (
-            estimated_clients * seconds_to_reset / self.remaining))
+        self.next_request_timestamp = min(
+            self.reset_timestamp,
+            now + max(min((seconds_to_reset - self.remaining) / 2, 10), 0),
+        )
